from django.contrib.auth.hashers import make_password
from django.contrib.auth.models import UserManager as DjangoUserManager


class UserManager(DjangoUserManager):
    def _create_user(self, email, password, **extra_fields):
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.password = make_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email, password=None, **extra_fields):
        extra_fields.setdefault("is_staff", False)
        extra_fields.setdefault("is_superuser", False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password, **kwargs):
        kwargs["is_staff"] = True
        kwargs["is_superuser"] = True
        kwargs["birth_date"] = "1930-01-01"
        kwargs["is_superuser"] = True
        return self.create_user(email, password, **kwargs)



import re

from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.utils.translation import gettext_lazy as _


class BRCPFValidator(RegexValidator):
    """
    Based on django-localflavor:
    https://github.com/django/django-localflavor/blob/master/localflavor/br/validators.py
    """

    regex = re.compile(r"^(\d{3})\.(\d{3})\.(\d{3})-(\d{2})$")
    message = _("CPF Inválido.")

    @staticmethod
    def calculate_verification_digit(value, _range):
        calculated_digit = (
            sum([i * int(value[idx]) for idx, i in enumerate(_range)]) % 11
        )

        if calculated_digit >= 2:
            return 11 - calculated_digit
        return 0

    def __call__(self, value):
        if not value.isdigit():
            cpf = self.regex.search(value)
            if cpf:
                value = "".join(cpf.groups())
            else:
                raise ValidationError(self.message, code="invalid")

        if len(value) != 11:
            raise ValidationError(self.message, code="max_digits")

        original_verification_digit = value[-2:]

        first_digit = self.calculate_verification_digit(value, range(10, 1, -1))
        value = value[:-2] + str(first_digit) + value[-1]

        second_digit = self.calculate_verification_digit(value, range(11, 1, -1))
        value = value[:-1] + str(second_digit)

        if value[-2:] != original_verification_digit:
            raise ValidationError(self.message, code="invalid")
        if value.count(value[0]) == 11:
            raise ValidationError(self.message, code="invalid")


class OnlyUpperCaseAndNumbersValidator(RegexValidator):
    regex = r"^[A-Z0-9]*$"
    message = "O código do cupom só pode conter números e letras maiúsculas"

    def __call__(self, value: str) -> None:
        if not re.search(self.regex, value):
            raise ValidationError(self.message, code="invalid")


class OnlyAlphabeticValidator(RegexValidator):
    regex = re.compile(
        r"^[A-Za-záàâåãäéëèêíìïîóôòõöüùúûçñÁÀÂÃÄÅÉÈËÊÍÌÏÎÓÔÕÖÒÙÚÛÜýÿÝŸÇÑèìîòûÈÌÎÒÛ ]+$"
    )
    message = "O campo não pode incluir números ou caracteres especiais"

    def __call__(self, value: str) -> None:
        if not self.regex.search(value):
            raise ValidationError(self.message, code="invalid")



from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.db.models import Q
from django.utils.translation import gettext as _

from lacrei_models.utils.models import BaseModel, HashedAutoField
from lacrei_models.utils.validators import (
    BRCPFValidator,
    OnlyUpperCaseAndNumbersValidator,
)

NULLABLE = {"null": True, "blank": True}


class Agreement(BaseModel):
    id = HashedAutoField(primary_key=True)
    user = models.ForeignKey(
        "client.User",
        on_delete=models.CASCADE,
        related_name="agreements",
        verbose_name=_("Usuário"),
    )
    name = models.CharField(
        max_length=100, verbose_name=_("Nome do Convênio e/ou plano")
    )
    user_name = models.CharField(
        max_length=100, verbose_name=_("Nome da pessoa beneficiária")
    )
    registration_number = models.CharField(
        max_length=50, verbose_name=_("Matrícula da pessoa beneficiária")
    )
    expiration_date = models.DateField(
        verbose_name=_("Validade (opcional)"), null=True, blank=True
    )
    user_cpf = models.CharField(
        max_length=50,
        verbose_name=_("CPF da pessoa beneficiária"),
        validators=[BRCPFValidator()],
    )

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=["name", "registration_number"],
                name="unique_name_registration_number",
            )
        ]
        ordering = ["-created_at"]
        app_label = "appointments"


class Appointment(BaseModel):
    PENDING = "pending"
    CONFIRMED = "confirmed"
    CANCELED = "canceled"
    COMPLETED = "completed"
    SCHEDULE_STATUS = [
        (PENDING, _("Agendamento Pendente")),
        (CONFIRMED, _("Confirmado")),
        (CANCELED, _("Cancelado")),
        (COMPLETED, _("Realizada")),
    ]
    IN_PERSON = "in_person"
    ONLINE = "online"
    APPOINTMENT_TYPE = [
        (IN_PERSON, _("Presencial")),
        (ONLINE, _("Online")),
    ]
    id = HashedAutoField(primary_key=True)
    professional = models.ForeignKey(
        "professional.Professional",
        on_delete=models.CASCADE,
        related_name="appointments",
        verbose_name=_("Profissional"),
    )
    user = models.ForeignKey(
        "client.User",
        on_delete=models.CASCADE,
        related_name="appointments",
        verbose_name=_("Usuário"),
    )
    date = models.DateTimeField(verbose_name=_("Data da Consulta"))
    status = models.CharField(
        choices=SCHEDULE_STATUS,
        default=PENDING,
        verbose_name=_("Status da Consulta"),
    )
    type = models.CharField(
        max_length=10,
        choices=APPOINTMENT_TYPE,
        default=IN_PERSON,
        verbose_name=_("Tipo da Consulta"),
    )
    rescheduled = models.BooleanField(
        default=False, verbose_name=("verificação de reagendamento da consulta")
    )
    google_calendar_event_id = models.CharField(
        max_length=50,
        verbose_name=_("ID de integração com o google calendar"),
        null=True,
    )
    external_reference = models.CharField(
        max_length=255,
        unique=True,
        null=True,
        blank=True,
        db_index=True,
        verbose_name=_("Referência Externa (Asaas)"),
    )
    agreement = models.ForeignKey(
        Agreement,
        on_delete=models.PROTECT,
        verbose_name=_("Convênio"),
        null=True,
        blank=True,
    )
    cancellation_policy_accepted = models.BooleanField(
        verbose_name="Aceito apolice de cancelamento", default=False
    )
    health_insurance_details = models.JSONField(
        verbose_name="Dados de pagamento convênio ou particular",
        default=dict,
        blank=True,
    )

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=("professional", "date"),
                condition=~Q(status="canceled"),
                name="unique_professional_appointment_non_canceled",
            ),
            models.UniqueConstraint(
                fields=("user", "date"),
                condition=~Q(status="canceled"),
                name="unique_user_appointment_non_canceled",
            ),
        ]
        app_label = "appointments"
        db_table = "lacreiid_appointment"


class Report(BaseModel):
    id = HashedAutoField(primary_key=True)
    appointment = models.ForeignKey(
        Appointment,
        on_delete=models.CASCADE,
        related_name="reports",
        verbose_name=_("Consulta"),
    )

    created_by_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.PROTECT,
        limit_choices_to={"model__in": ["user", "professional"]},
        verbose_name=_("Tipo de Usuário"),
        null=True,
    )
    created_by_object_id = models.CharField(
        max_length=32, verbose_name=_("ID de Usuário"), null=True
    )
    created_by = GenericForeignKey("created_by_content_type", "created_by_object_id")

    feedback = models.CharField(max_length=255, verbose_name=_("Feedback da consulta"))
    eval = models.IntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(5)],
        verbose_name=_("Nota da consulta"),
    )

    class Meta:
        app_label = "appointments"
        db_table = "lacreiid_report"


class Cancellation(BaseModel):
    id = HashedAutoField(primary_key=True)
    appointment = models.ForeignKey(
        Appointment,
        on_delete=models.CASCADE,
        related_name="cancellations",
        verbose_name=_("Consulta"),
    )

    created_by_content_type = models.ForeignKey(
        ContentType,
        on_delete=models.PROTECT,
        limit_choices_to={"model__in": ["user", "professional"]},
        verbose_name=_("Tipo de Usuário"),
        null=True,
    )
    created_by_object_id = models.CharField(
        max_length=32, verbose_name=_("ID de Usuário"), null=True
    )
    created_by = GenericForeignKey("created_by_content_type", "created_by_object_id")

    reason = models.CharField(max_length=255, verbose_name=_("Motivo"))

    class Meta:
        app_label = "appointments"
        db_table = "lacreiid_cancellation"


class Partner(BaseModel):
    id = HashedAutoField(primary_key=True)
    name = models.CharField(
        max_length=50,
        verbose_name=_("Nome do Parceiro"),
    )

    def __str__(self):
        return self.name

    class Meta:
        app_label = "appointments"


class CouponManager(models.Manager):
    pass


class Coupon(BaseModel):
    id = HashedAutoField(primary_key=True)
    code = models.CharField(
        max_length=20,
        validators=[OnlyUpperCaseAndNumbersValidator()],
        verbose_name=_("Código do cupom"),
    )
    discount = models.IntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(100)],
        verbose_name=_("Porcentagem de desconto"),
    )
    limit_date = models.DateField(
        null=True, blank=True, verbose_name=_("Validade do cupom")
    )
    usages_limit = models.IntegerField(
        validators=[MinValueValidator(1)], verbose_name=_("Limite de usos")
    )
    usages = models.IntegerField(default=0, verbose_name=_("Usos"))
    partner = models.ForeignKey(
        Partner,
        on_delete=models.SET_NULL,
        verbose_name=_("Nome do parceiro"),
        null=True,
        blank=True,
    )

    objects = CouponManager()

    class Meta:
        app_label = "appointments"
        db_table = "lacreiid_coupon"



from django.db import models
from django.utils.translation import gettext_lazy as _


class EthnicGroupChoices(models.TextChoices):
    """
    Representa as opções fixas para o Grupo Étnico.
    """

    AMARELA = "AM", _("Amarela")
    BRANCA = "BR", _("Branca")
    INDIGENA = "IN", _("Indígena")
    PARDA = "PA", _("Parda")
    PRETA = "PR", _("Preta")
    OUTRO = "OU", _("Outro (Especificar)")


class GenderIdentityChoices(models.TextChoices):
    """
    Representa as opções fixas para a Identidade de Gênero.
    """

    AGENERA = "AG", _("Agênera")
    FLUIDA = "FL", _("Fluida")
    HOMEM_CIS = "HC", _("Homem Cisgênero")
    HOMEM_TRANS = "HT", _("Homem Transgênero")
    INTERSSEXO = "IT", _("Interssexo")
    MULHER_CIS = "MC", _("Mulher Cisgênero")
    MULHER_TRANS = "MT", _("Mulher Transgênero")
    NAO_BINARIA = "NB", _("Não-binária")
    TRAVESTI = "TV", _("Travesti")
    OUTRO = "OU", _("Outra (Especificar)")


class SexualityChoices(models.TextChoices):
    """
    Representa as opções fixas para a Sexualidade.
    """

    ASSEXUAL = "AS", _("Assexual")
    BISSEXUAL = "BI", _("Bissexual")
    DEMISSEXUAL = "DE", _("Demissexual")
    GAY = "GA", _("Gay")
    HETEROSSEXUAL = "HT", _("Heterossexual")
    PANSEXUAL = "PA", _("Pansexual")
    LESBICA = "LE", _("Lésbica")
    OUTRO = "OU", _("Outra (Especificar)")


class PronounChoices(models.TextChoices):
    """
    Representa as opções fixas para Pronome, incluindo o Artigo para
    a lógica do 'display_article'.
    """

    ELE_DELE = "EL", _("Ele/Dele")
    ELA_DELA = "EA", _("Ela/Dela")
    ELU_DELU = "EU", _("Elu/Delu")
    OUTRO = "OU", _("Outro")

    def get_article(self):
        """Retorna o terceiro elemento da tupla de escolha (o artigo)."""
        try:
            return self._value_[2]
        except IndexError:
            return None


class DisabilityTypeChoices(models.TextChoices):
    """
    Representa as opções fixas para Tipos de Deficiência (seleção múltipla).
    """

    NAO_POSSUI = "NP", _("Não possuo deficiência")
    AUDITIVA = "AU", _("Auditiva")
    COGNITIVA = "CO", _("Cognitiva")
    MOTORA = "MO", _("Motora")
    MULTIPLA = "MU", _("Múltipla")
    VISUAL = "VI", _("Visual")
    OUTRA = "OU", _("Outra")


class AnswerType(models.TextChoices):
    """
    Representa as opções fixas para os campos onde são permitidos outros.
    """

    ETHNIC_GROUP = "EG", _("Grupo Étnico")
    GENDER_IDENTITY = "GI", _("Identidade de Gênero")
    SEXUALITY = "SE", _("Sexualidade")
    PRONOUN = "PR", _("Pronome")
    DISABILITY = "DS", _("Deficiência")



from django.db import models
from django.utils.translation import gettext as _

from lacrei_models.utils.models import BaseModel, HashedAutoField

NULLABLE = {"null": True, "blank": True}


class GoogleAccount(BaseModel):

    id = HashedAutoField(primary_key=True, verbose_name=_("ID"))
    user = models.ForeignKey(
        "client.User",
        on_delete=models.CASCADE,
        related_name="google_accounts",
        verbose_name=_("Usuário"),
    )
    access_token = models.TextField(verbose_name=_("Token de Acesso"))
    refresh_token = models.TextField(verbose_name=_("Token de Atualização"))
    expires_at = models.DateTimeField(verbose_name=_("Expira em"))
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_("Criado em"))

    class Meta:
        db_table = "lacreiid_google_account"
        verbose_name = _("Conta Google")
        verbose_name_plural = _("Contas Google")
        app_label = "sync"

    def __str__(self):
        return f"Google Account ({self.user.email})"


class GoogleCalendarEvent(BaseModel):
    id = HashedAutoField(primary_key=True)
    user = models.ForeignKey(
        "client.User",
        on_delete=models.CASCADE,
        verbose_name=_("Usuário"),
    )
    event_id = models.CharField(max_length=255, verbose_name=_("ID do Evento Google"))
    summary = models.CharField(max_length=255, verbose_name=_("Titulo"))
    description = models.TextField(**NULLABLE, verbose_name=_("Descrição"))
    start_time = models.DateTimeField(verbose_name=_("Início dop Evento"))
    end_time = models.DateTimeField(verbose_name=_("Fim do Evento"))
    created_at = models.DateTimeField(auto_now_add=True, verbose_name=_("Criado em"))
    updated_at = models.DateTimeField(auto_now=True, verbose_name=_("Atualizado em"))

    class Meta:
        db_table = "lacreiid_google_calendar_event"
        verbose_name = _("Evento do Google Calendar")
        verbose_name_plural = _("Eventos do Google Calendar")
        app_label = "sync"

    def __str__(self):
        return f"{self.summary} ({self.start_time})"



import logging

from django.utils.functional import cached_property
from django.utils.translation import gettext as _

from lacrei_models.utils.social_choices import (
    AnswerType,
    EthnicGroupChoices,
    GenderIdentityChoices,
    PronounChoices,
    SexualityChoices,
)

PROFESSIONAL_VALIDATION_PENDING_ADMIN = _("Pendente Lacrei")
PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT = _("Pendente Profissional")
PROFESSIONAL_ALREADY_APPROVED_FOR_STEP = _("Aprovado")

logger = logging.getLogger(__name__)


class VerificationStep:
    name: str
    url_path_to_submit: str
    url_path_to_wait_completion: str
    description: str

    def __init__(self, professional):
        self.professional = professional

    @cached_property
    def is_completed(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def is_submitted(self):  # pragma: no cover
        raise NotImplementedError

    @cached_property
    def redirect_to(self):
        if self.is_completed:
            return None

        if self.is_submitted:
            return self.url_path_to_wait_completion

        return self.url_path_to_submit

    def to_representation(self):
        return {
            "is_completed": self.is_completed,
            "is_submitted": self.is_submitted,
            "redirect_to": self.redirect_to,
            "description": self.description,
            "internal_message": (
                PROFESSIONAL_ALREADY_APPROVED_FOR_STEP
                if self.is_completed
                else (
                    PROFESSIONAL_VALIDATION_PENDING_DATA_INPUT
                    if not self.is_submitted
                    else PROFESSIONAL_VALIDATION_PENDING_ADMIN
                )
            ),
        }


class BoardRegistrationNumber(VerificationStep):
    """
    Uma pessoa interna da lacrei irá validar o número do conselho no respectivo portal."

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: apagamos o cadastro e pedimos para refazer, com um número certo.
    - Se aprovado: segue para o pós cadastro.
    """

    name = "board_registration_number"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "1ª - Validação no conselho"

    @cached_property
    def is_completed(self):
        return self.professional.profile_status == "approved"

    @cached_property
    def is_submitted(self):
        return True


class EmailConfirmation(VerificationStep):
    """
    Nessa etapa, uma pessoa interna da Lacrei já aprovou o cadastro,
    e agora aguardamos a pessoa profissional confirmar o email.
    """

    name = "email_confirmation"
    url_path_to_submit = "/"
    url_path_to_wait_completion = "/saude/verificacao-inscricao/"
    description = "2ª - Confirmação do email"

    @cached_property
    def is_completed(self):
        return self.professional.user.email_verified

    @cached_property
    def is_submitted(self):
        # Se a etapa anterior (BoardRegistrationNumber) foi aprovada,
        # então esta etapa está automaticamente "submitted"
        return BoardRegistrationNumber(self.professional).is_completed


class PostRegistrationData(VerificationStep):
    """
    Nessa passo a pessoa profissional irá enviar os dados restantes para o cadastro.

    Após validação, será enviado um email de confirmação, onde:
    - Se rejeitado: mandamos email pedindo para reenviar os dados.
    - Se aprovado: Liberado para ficar ativo na plataforma
    """

    name = "post_registration_data"
    url_path_to_submit = "/saude/cadastro-dados-pessoais/"
    url_path_to_wait_completion = "/saude/painel-cadastro-analise/"
    description = "4ª - Pós cadastro"

    @cached_property
    def is_completed(self):
        return self.professional.active

    @cached_property
    def is_submitted(self):
        return hasattr(self.professional, "clinic") and bool(
            self.professional.board_certification_selfie
        )


class IntersectionalityData(VerificationStep):
    """
    Na etapa, pedimos que adicione os dados de interseccionalidade.
    """

    name = "add_intersectionality_data"
    url_path_to_submit = "/saude/cadastro-diversidade/"
    url_path_to_wait_completion = ""
    description = "3ª - Diversidade"

    # Função auxiliar para verificar se 'Outro' foi preenchido
    def _has_other_answer(self, field_value, answer_type_enum):
        """Verifica se o campo tem o código 'OU' E se existe um registro em OtherProfileAnswer."""
        if field_value == EthnicGroupChoices.OUTRO.value:  # Usa .value para CharField
            try:
                # Usa o related_manager 'other_answers' para buscar a resposta
                return self.professional.other_answers.filter(
                    answer_type=answer_type_enum.value
                ).exists()
            except Exception:
                return False
        return False

    @cached_property
    def is_completed(self):
        professional = self.professional

        ethnic_check = bool(professional.ethnic_group) and (
            professional.ethnic_group != EthnicGroupChoices.OUTRO.value
            or self._has_other_answer(
                professional.ethnic_group, AnswerType.ETHNIC_GROUP
            )
        )

        gender_check = bool(professional.gender_identity) and (
            professional.gender_identity != GenderIdentityChoices.OUTRO.value
            or self._has_other_answer(
                professional.gender_identity, AnswerType.GENDER_IDENTITY
            )
        )

        sexuality_check = bool(professional.sexuality) and (
            professional.sexuality != SexualityChoices.OUTRO.value
            or self._has_other_answer(professional.sexuality, AnswerType.SEXUALITY)
        )

        pronoun_check = bool(professional.pronoun) and (
            professional.pronoun != PronounChoices.OUTRO.value
            or self._has_other_answer(professional.pronoun, AnswerType.PRONOUN)
        )

        disability_check = bool(professional.disability_types)

        return all(
            [
                ethnic_check,
                gender_check,
                sexuality_check,
                pronoun_check,
                disability_check,
            ]
        )

    @cached_property
    def is_submitted(self):
        return self.is_completed


class VerificationStepsService:
    def __init__(self, professional):
        self.professional = professional
        self.steps = [
            BoardRegistrationNumber(professional),
            EmailConfirmation(professional),
            IntersectionalityData(professional),
            PostRegistrationData(professional),
        ]

    def to_representation(self) -> dict:
        current_step = None
        steps_repr = {}

        for step in self.steps:
            if not step.is_completed and not current_step:
                current_step = step.name

            steps_repr[step.name] = step.to_representation()

        return {"current_step": current_step, "steps": steps_repr}

    def current_step(self):
        representation = self.to_representation()
        if not representation["current_step"]:
            return None
        return representation["steps"][representation["current_step"]]



import uuid
from decimal import Decimal

from django.core.validators import MinValueValidator
from django.db import models
from django.utils.translation import gettext as _

from lacrei_models.appointments.models import Appointment
from lacrei_models.professional.models import Professional
from lacrei_models.utils.models import BaseModel, HashedAutoField


class Bank(BaseModel):
    code = models.CharField(max_length=10, unique=True)
    name = models.CharField(max_length=255)
    ispb = models.CharField(max_length=20, null=True, blank=True)

    class Meta:
        app_label = "payments"


class Payment(BaseModel):
    PENDING = "pending"
    PAYED = "payed"
    FAILED = "failed"
    PAYMENT_STATUS = [
        (PENDING, _("Pendente")),
        (PAYED, _("Pago")),
        (FAILED, _("Falhou")),
    ]
    CREDIT_CARD = "credit_card"
    PIX = "pix"
    PAYMENT_METHOD = [
        (CREDIT_CARD, _("Cartão de Crédito")),
        (PIX, _("PIX")),
    ]

    REFUND_STATUS = [
        ("none", _("Sem reembolso")),
        ("requested", _("Solicitado")),
        ("processing", _("Processando")),
        ("processed", _("Processado")),
        ("failed", _("Falhou")),
        ("no_refund", _("Sem direito a reembolso")),
    ]

    id = HashedAutoField(primary_key=True)
    appointment = models.ForeignKey(
        Appointment,
        on_delete=models.CASCADE,
        related_name="payments",
        verbose_name=_("Consulta"),
    )
    asaas_id = models.CharField(
        max_length=255,
        unique=True,
        null=True,
        blank=True,
        db_index=True,
        verbose_name=_("ID da Cobrança na Asaas"),
    )
    value = models.DecimalField(
        max_digits=6,
        decimal_places=2,
        validators=[MinValueValidator(Decimal(0.00))],
        verbose_name=_("Valor da consulta"),
    )
    status = models.CharField(
        max_length=10,
        choices=PAYMENT_STATUS,
        default=PENDING,
        verbose_name=_("Status do Pagamento"),
    )
    method = models.CharField(
        max_length=15,
        choices=PAYMENT_METHOD,
        default=CREDIT_CARD,
        verbose_name=_("Método do Pagamento"),
    )
    external_id = models.CharField(
        max_length=100,
        null=True,
        blank=True,
        verbose_name=_("ID externo (ASAAS)"),
    )
    transferred_payment = models.BooleanField(default=False)

    class Meta:
        app_label = "payments"
        indexes = [
            models.Index(fields=["external_id"], name="payment_external_id_idx"),
            models.Index(
                fields=["status", "created_at"], name="payment_status_created_idx"
            ),
        ]

    refund_status = models.CharField(
        max_length=20,
        choices=REFUND_STATUS,
        default="none",
        verbose_name=_("Status do Reembolso"),
    )
    refund_reason = models.TextField(
        blank=True, null=True, verbose_name=_("Motivo do reembolso")
    )
    refund_requested_at = models.DateTimeField(
        blank=True, null=True, verbose_name=_("Data da solicitação do reembolso")
    )
    refund_requested_by = models.ForeignKey(
        "client.User",
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name="requested_refunds",
        verbose_name=_("Usuário que solicitou o reembolso"),
    )
    external_refund_id = models.CharField(
        max_length=100,
        blank=True,
        null=True,
        verbose_name=_("ID do reembolso no Asaas"),
    )
    patient_refund_amount = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        default=Decimal("0.00"),
        verbose_name=_("Valor reembolsado ao paciente"),
    )
    professional_amount = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        default=Decimal("0.00"),
        verbose_name=_("Valor do profissional"),
    )
    lacrei_amount = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        default=Decimal("0.00"),
        verbose_name=_("Valor Lacrei"),
    )

    def __str__(self):
        return f"Payment {self.id} - {self.appointment} - R$ {self.value}"


class Customer(models.Model):
    id = models.CharField(primary_key=True, max_length=100)
    id_user = models.ForeignKey("client.User", on_delete=models.CASCADE)

    class Meta:
        app_label = "payments"


class BillingStatus(BaseModel):
    # ... (Modelo igual)
    PENDING = "PENDING"
    RECEIVED = "RECEIVED"
    CONFIRMED = "CONFIRMED"
    OVERDUE = "OVERDUE"
    REFUNDED = "REFUNDED"
    RECEIVED_IN_CASH = "RECEIVED_IN_CASH"
    REFUND_REQUESTED = "REFUND_REQUESTED"
    REFUND_IN_PROGRESS = "REFUND_IN_PROGRESS"
    CHARGEBACK_REQUESTED = "CHARGEBACK_REQUESTED"
    CHARGEBACK_DISPUTE = "CHARGEBACK_DISPUTE"
    AWAITING_CHARGEBACK_REVERSAL = "AWAITING_CHARGEBACK_REVERSAL"
    DUNNING_REQUESTED = "DUNNING_REQUESTED"
    DUNNING_RECEIVED = "DUNNING_RECEIVED"
    AWAITING_RISK_ANALYSIS = "AWAITING_RISK_ANALYSIS"

    STATUS_CHOICES = [
        (PENDING, _("Pendente")),
        (RECEIVED, _("Recebido")),
        (CONFIRMED, _("Confirmado")),
        (OVERDUE, _("Vencido")),
        (REFUNDED, _("Reembolsado")),
        (RECEIVED_IN_CASH, _("Recebido em dinheiro")),
        (REFUND_REQUESTED, _("Reembolso solicitado")),
        (REFUND_IN_PROGRESS, _("Reembolso em andamento")),
        (CHARGEBACK_REQUESTED, _("Chargeback solicitado")),
        (CHARGEBACK_DISPUTE, _("Disputa de chargeback")),
        (AWAITING_CHARGEBACK_REVERSAL, _("Aguardando reversão de chargeback")),
        (DUNNING_REQUESTED, _("Cobrança judicial solicitada")),
        (DUNNING_RECEIVED, _("Cobrança judicial recebida")),
        (AWAITING_RISK_ANALYSIS, _("Aguardando análise de risco")),
    ]

    id = HashedAutoField(primary_key=True)
    status = models.CharField(max_length=30, choices=STATUS_CHOICES)
    payment_id = models.ForeignKey("Payment", on_delete=models.CASCADE)

    def __str__(self):
        return f"BillingStatus {self.id} - {self.status}"

    class Meta:
        app_label = "payments"


class PaymentsRelease(models.Model):
    appointment = models.ForeignKey(
        Appointment,
        on_delete=models.CASCADE,
        related_name="payment_transaction",
        verbose_name=_("Consulta"),
    )
    professional = models.ForeignKey(
        Professional,
        on_delete=models.CASCADE,
        related_name="professional_payment_transaction",
        verbose_name=_("Consulta"),
    )
    amount = models.DecimalField(max_digits=10, decimal_places=2)
    status = models.CharField(max_length=20)

    asaas_transfer_id = models.CharField(max_length=100, blank=True, null=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        app_label = "payments"


class IdempotencyKey(BaseModel):
    key = models.CharField(max_length=128, unique=True)
    appointment_id = models.CharField(max_length=64)
    method = models.CharField(max_length=15)
    value = models.DecimalField(max_digits=10, decimal_places=2)
    response_status = models.IntegerField(null=True, blank=True)
    response_body = models.JSONField(null=True, blank=True)
    expires_at = models.DateTimeField()

    class Meta:
        app_label = "payments"
        indexes = [
            models.Index(fields=["expires_at"], name="idempotency_key_expires_idx"),
        ]


class PaymentReprocess(BaseModel):
    PENDING = "pending"
    SCHEDULED = "scheduled"
    COMPLETED = "completed"
    FAILED = "failed"
    STATUS = [
        (PENDING, PENDING),
        (SCHEDULED, SCHEDULED),
        (COMPLETED, COMPLETED),
        (FAILED, FAILED),
    ]

    payment = models.ForeignKey(
        Payment, on_delete=models.CASCADE, related_name="reprocesses"
    )
    external_id = models.CharField(max_length=100, null=True, blank=True)
    attempt_count = models.IntegerField(default=0)
    last_error = models.TextField(null=True, blank=True)
    next_attempt_at = models.DateTimeField(null=True, blank=True)
    status = models.CharField(max_length=12, choices=STATUS, default=PENDING)

    class Meta:
        app_label = "payments"
        indexes = [
            models.Index(
                fields=["status", "next_attempt_at"], name="reprocess_status_next_idx"
            ),
        ]


class AsaasWebhookLog(models.Model):
    RECEIVED = "RECEIVED"
    QUEUED = "QUEUED"
    PROCESSING = "PROCESSING"
    SUCCESS = "SUCCESS"
    FAILED = "FAILED"
    PROCESSED = "PROCESSED"
    IGNORED = "IGNORED"

    STATUS_CHOICES = [
        (RECEIVED, "Recebido"),
        (QUEUED, "Enfileirado"),
        (PROCESSING, "Processando"),
        (SUCCESS, "Sucesso"),
        (FAILED, "Falhou"),
        (PROCESSED, "Processado"),
        (IGNORED, "Ignorado"),
    ]

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    event_id = models.CharField(
        max_length=255, unique=True, help_text="ID do evento da Asaas para idempotência"
    )
    event_type = models.CharField(max_length=100, db_index=True)
    payload = models.JSONField()
    status = models.CharField(
        max_length=20, choices=STATUS_CHOICES, default=RECEIVED, db_index=True
    )
    received_at = models.DateTimeField(auto_now_add=True)
    processing_log = models.TextField(
        blank=True, null=True, help_text="Log de erros ou observações do processamento."
    )

    class Meta:
        app_label = "payments"
        verbose_name = "Log de Webhook da Asaas"
        verbose_name_plural = "Logs de Webhooks da Asaas"
        ordering = ["-received_at"]

    def __str__(self):
        return f"{self.event_type} ({self.id}) - {self.status}"



from allauth.account.models import EmailAddress
from django.contrib.auth.models import AbstractUser
from django.contrib.postgres.fields import ArrayField
from django.db import models
from django.utils.translation import gettext as _
from phonenumber_field.modelfields import PhoneNumberField

from lacrei_models.client.managers import UserManager
from lacrei_models.utils.models import (
    NULLABLE,
    BaseModel,
    HashedAutoField,
    HashedFileName,
)
from lacrei_models.utils.social_choices import (
    AnswerType,
    DisabilityTypeChoices,
    EthnicGroupChoices,
    GenderIdentityChoices,
    PronounChoices,
    SexualityChoices,
)
from lacrei_models.utils.validators import OnlyAlphabeticValidator


class User(AbstractUser, BaseModel):
    USER = "user"
    PROFESSIONAL = "professional"
    LOGGED_AS_CHOICES = [(USER, _("Usuário")), (PROFESSIONAL, _("Profissional"))]
    id = HashedAutoField(primary_key=True)
    logged_as = models.CharField(
        max_length=12,
        choices=LOGGED_AS_CHOICES,
        null=True,
        blank=True,
        help_text=_("Indica se o usuário está logado como Usuário ou Profissional"),
    )
    email = models.EmailField(unique=True)
    first_name = models.CharField(
        max_length=150, blank=False, validators=[OnlyAlphabeticValidator()]
    )
    last_name = models.CharField(
        max_length=150, blank=False, validators=[OnlyAlphabeticValidator()]
    )
    birth_date = models.DateField(**NULLABLE)
    is_18_years_old_or_more = models.BooleanField(**NULLABLE)
    last_login = models.DateTimeField(auto_now_add=True)
    email_verified = models.BooleanField(default=False)

    accepted_privacy_document = models.BooleanField(default=False)
    privacy_document = models.ForeignKey(
        "client.PrivacyDocument", on_delete=models.PROTECT, null=True, blank=False
    )
    newsletter_subscribed = models.BooleanField(default=True)

    phone = PhoneNumberField(null=True, blank=True)
    phone_verified = models.BooleanField(default=False)
    phone_verification_token = models.CharField(max_length=6, **NULLABLE)
    phone_verification_token_expires_at = models.DateTimeField(**NULLABLE)

    objects = UserManager()
    username = None
    date_joined = None
    USERNAME_FIELD = "email"
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _("Pessoa Usuária")
        verbose_name_plural = _("Pessoas Usuárias")
        app_label = "client"
        db_table = "lacreiid_user"

    def post_create_instance(self, *args, **kwargs):
        self.profile = Profile.objects.create(user=self)


class PrivacyDocument(BaseModel):
    privacy_policy = models.URLField()
    terms_of_use = models.URLField()
    profile_type = models.CharField(
        max_length=20,
        choices=[("client", _("Lacrei ID")), ("professional", _("Lacrei Saúde"))],
    )

    class Meta:
        verbose_name = _("Termo de uso e privacidade")
        verbose_name_plural = _("Termos de uso e privacidade")
        app_label = "client"
        db_table = "lacreiid_privacydocument"

    def __str__(self):
        return f"{self.id} - {self.profile_type} - {self.created_at}"


class BaseProfile(BaseModel):
    ethnic_group = models.CharField(
        max_length=2,
        choices=EthnicGroupChoices.choices,
        **NULLABLE,
        verbose_name=_("Grupo Étnico"),
    )
    gender_identity = models.CharField(
        max_length=2,
        choices=GenderIdentityChoices.choices,
        **NULLABLE,
        verbose_name=_("Identidade de Gênero"),
    )
    sexuality = models.CharField(
        max_length=2,
        choices=SexualityChoices.choices,
        **NULLABLE,
        verbose_name=_("Sexualidade"),
    )
    pronoun = models.CharField(
        max_length=2,
        choices=PronounChoices.choices,
        **NULLABLE,
        verbose_name=_("Pronome"),
    )
    disability_types = ArrayField(
        models.CharField(
            max_length=2,
            choices=DisabilityTypeChoices.choices,
        ),
        **NULLABLE,
        verbose_name=_("Tipos de deficiência"),
    )

    def _get_other_value(self, answer_type_code):
        """Função auxiliar para buscar o objeto 'Outro' no banco de dados,
        navegando via User para a instância Profile (Client).
        """
        try:

            return self.user.profile.other_answers.get(answer_type=answer_type_code)

        except Exception:
            return None

    @property
    def display_ethnic_group(self):
        if self.ethnic_group == EthnicGroupChoices.OUTRO:
            other_answer = self._get_other_value(AnswerType.ETHNIC_GROUP)
            return other_answer.custom_value if other_answer else None
        return self.ethnic_group or None

    @property
    def display_gender_identity(self):
        if self.gender_identity == GenderIdentityChoices.OUTRO:
            other_answer = self._get_other_value(AnswerType.GENDER_IDENTITY.value)
            return other_answer.custom_value if other_answer else None
        return self.gender_identity or None

    @property
    def display_sexuality(self):
        if self.sexuality == SexualityChoices.OUTRO:
            other_answer = self._get_other_value(AnswerType.SEXUALITY.value)
            return other_answer.custom_value if other_answer else _("Outro")

        return self.sexuality or None

    @property
    def display_pronoun(self):
        if self.pronoun == PronounChoices.OUTRO:
            other_answer = self._get_other_value(AnswerType.PRONOUN.value)

            return other_answer.custom_value if other_answer else None
        return self.pronoun or None

    @property
    def display_article(self):
        if self.pronoun == PronounChoices.OUTRO:
            other_answer = self._get_other_value(
                OtherProfileAnswer.AnswerType.PRONOUN.value
            )

            if other_answer and other_answer.custom_value:
                return other_answer.custom_value[-1].lower()
            return None

        try:
            PRONOUN_ARTICLE_MAP = {
                PronounChoices.ELE_DELE.value: "o",
                PronounChoices.ELA_DELA.value: "a",
                PronounChoices.ELU_DELU.value: "e",
            }
            return PRONOUN_ARTICLE_MAP.get(self.pronoun)

        except Exception:
            return None

    @property
    def display_disability_types(self):
        codigos_selecionados = self.disability_types or []

        labels = []
        other_answer = None

        if DisabilityTypeChoices.OUTRA.value in codigos_selecionados:
            other_answer = self._get_other_value(AnswerType.DISABILITY)

        CODIGOS_FIXOS = [
            DisabilityTypeChoices.NAO_POSSUI.value,
            DisabilityTypeChoices.AUDITIVA.value,
            DisabilityTypeChoices.COGNITIVA.value,
            DisabilityTypeChoices.MOTORA.value,
            DisabilityTypeChoices.MULTIPLA.value,
            DisabilityTypeChoices.VISUAL.value,
        ]

        for codigo in codigos_selecionados:
            try:
                if codigo in CODIGOS_FIXOS:
                    labels.append(codigo)

                elif codigo == DisabilityTypeChoices.OUTRA.value:
                    if other_answer and other_answer.custom_value:
                        labels.append(other_answer.custom_value)
                    else:
                        label = str(DisabilityTypeChoices(codigo).label)
                        labels.append(label)

                else:
                    label = str(DisabilityTypeChoices(codigo).label)
                    labels.append(label)

            except ValueError:
                continue

        return labels if labels else None

    class Meta:
        abstract = True


class OtherProfileAnswer(models.Model):
    user = models.ForeignKey(
        "client.User",
        on_delete=models.CASCADE,
        related_name="user_other_answers",
        verbose_name=_("pessoa usuária"),
    )
    profile = models.ForeignKey(
        "client.Profile",
        on_delete=models.CASCADE,
        related_name="other_answers",
        verbose_name=_("Perfil Relacionado"),
    )
    answer_type = models.CharField(
        max_length=2, choices=AnswerType.choices, verbose_name=_("Tipo de Resposta")
    )
    custom_value = models.CharField(max_length=100, verbose_name=_("Valor Customizado"))

    class Meta:
        verbose_name = _("Resposta 'Outra' do épico de pós cadastro")
        verbose_name_plural = _("Respostas 'Outras' do épico de pós cadastro")
        # Garante que só haja uma resposta 'Outra' por tipo por perfil
        unique_together = ("profile", "answer_type")
        app_label = "client"
        db_table = "lacreiid_otherprofileanswer"


class Profile(BaseProfile):
    id = HashedAutoField(primary_key=True)
    user = models.OneToOneField(
        User, on_delete=models.PROTECT, verbose_name=_("Lacrei ID")
    )
    completed = models.BooleanField(
        default=False,
        verbose_name=_("Perfil completo"),
    )
    photo = models.ImageField(
        null=True,
        upload_to=HashedFileName("profile_photos"),
        verbose_name=_("Foto de perfil"),
    )
    photo_description = models.TextField(
        null=True, blank=True, help_text=_("Descrição da foto")
    )

    def __str__(self):
        try:
            return self.user.first_name
        except User.DoesNotExist:
            return "Perfil sem usuário"

    class Meta:
        verbose_name = _("Perfil")
        verbose_name_plural = _("Perfis")
        app_label = "client"
        db_table = "lacreiid_profile"


class CustomEmailAddress(EmailAddress):
    class Meta:
        proxy = True
        app_label = "client"
        db_table = "lacreiid_custom_email_address"


class LegacyProfileData(BaseModel):
    """
    Modelo de backup para armazenar dados de diversidade do Profile em formato de TEXTO
    (name/pronoun) antes da refatoração para Enums.
    """

    profile = models.OneToOneField(
        "Profile",
        on_delete=models.CASCADE,
        related_name="legacy_data",
        verbose_name=_("Perfil Original"),
    )

    ethnic_group_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Grupo Étnico (Nome Legado)"),
    )

    gender_identity_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Identidade de Gênero (Nome Legado)"),
    )

    sexual_orientation_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Orientação Sexual (Nome Legado)"),
    )

    pronoun_text = models.CharField(
        max_length=50, **NULLABLE, verbose_name=_("Pronome (Texto Legado)")
    )

    article_text = models.CharField(
        max_length=1, **NULLABLE, verbose_name=_("Artigo (Texto Legado)")
    )

    disability_types_names = models.TextField(
        **NULLABLE, verbose_name=_("Tipos de Deficiência (Nomes, LEGADO)")
    )

    other_ethnic_group = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Grupo Étnico (LEGADO)")
    )
    other_gender_identity = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Outra Identidade de Gênero (LEGADO)"),
    )
    other_sexual_orientation = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outra Orientação Sexual (LEGADO)")
    )
    other_pronoun = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Pronome (LEGADO)")
    )
    other_disability_types = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Outro Tipos de deficiência (LEGADO)"),
    )
    other_article = models.CharField(
        max_length=1, **NULLABLE, verbose_name=_("Outro Artigo (LEGADO)")
    )

    class Meta:
        verbose_name = _("Dados Legados do Perfil")
        verbose_name_plural = _("Dados Legados do Perfil")
        app_label = "client"
        db_table = "lacreiid_legacyprofiledata"



from django.db import models
from django.utils.translation import gettext_lazy as _

from lacrei_models.utils.models import NULLABLE, BaseModel


class Country(BaseModel):
    name = models.CharField(max_length=50)
    code = models.CharField(max_length=2)
    active = models.BooleanField(default=False)

    class Meta:
        verbose_name = _("País")
        verbose_name_plural = _("Países")
        app_label = "address"

    def __str__(self):
        return self.name


class State(BaseModel):
    name = models.CharField(max_length=50)
    code = models.CharField(max_length=2)
    ibge_code = models.IntegerField(**NULLABLE)
    active = models.BooleanField(default=False)
    country = models.ForeignKey(Country, on_delete=models.PROTECT)

    class Meta:
        verbose_name = _("Estado")
        verbose_name_plural = _("Estados")
        app_label = "address"

    def __str__(self):
        return self.name


class City(BaseModel):
    name = models.CharField(max_length=100)
    state = models.ForeignKey(State, on_delete=models.PROTECT)
    ibge_code = models.IntegerField(**NULLABLE)
    active = models.BooleanField(default=False)

    class Meta:
        verbose_name = _("Cidade")
        verbose_name_plural = _("Cidades")
        app_label = "address"

    def __str__(self):
        return self.name


class Neighborhood(BaseModel):
    name = models.CharField(max_length=100)
    city = models.ForeignKey(City, on_delete=models.PROTECT)

    class Meta:
        verbose_name = _("Bairro")
        verbose_name_plural = _("Bairros")
        app_label = "address"

    def __str__(self):
        return self.name


class ZipCode(BaseModel):
    code = models.CharField(max_length=100)
    address_line_1 = models.CharField(
        max_length=200, help_text=_("Logradouro"), **NULLABLE
    )
    address_line_2 = models.CharField(
        max_length=200, help_text=_("Complemento"), **NULLABLE
    )
    ibge_code = models.CharField(
        max_length=50, help_text=_("Código do IBGE"), **NULLABLE
    )
    neighborhood_name = models.CharField(
        max_length=200, help_text=_("Nome do bairro"), **NULLABLE
    )
    city = models.ForeignKey(City, on_delete=models.PROTECT, null=True, blank=False)
    neighborhood = models.ForeignKey(
        Neighborhood,
        on_delete=models.PROTECT,
        related_name="zip_codes",
        null=True,
        blank=True,
    )

    class Meta:
        verbose_name = _("CEP")
        verbose_name_plural = _("CEPs")
        app_label = "address"

    def __str__(self):
        return self.code



from django.conf import settings
from django.contrib.postgres.fields import ArrayField
from django.core.serializers.json import DjangoJSONEncoder
from django.db import models
from django.utils.translation import gettext as _

from lacrei_models.utils.models import BaseModel, HashedAutoField

NOTIFICATION_STATUS = [
    ("pending", _("Envio pendente")),
    ("success", _("Sucesso")),
    ("error", _("Erro ao enviar")),
]


class Notification(BaseModel):
    id = HashedAutoField(primary_key=True)
    template_prefix = models.CharField(max_length=250)
    send_to = ArrayField(
        models.EmailField(),
        help_text=_("Lista de destinatários (email)"),
        verbose_name=_("Destinatários"),
    )
    subject = models.CharField(max_length=500, verbose_name=_("Assunto"))
    pickled_context = models.BinaryField(
        verbose_name=_("Contexto da mensagem"), null=True
    )
    json_context = models.JSONField(
        default=dict,
        encoder=DjangoJSONEncoder,
        verbose_name=_("Contexto da mensagem como JSON"),
    )
    status = models.CharField(
        choices=NOTIFICATION_STATUS, default="PENDING", max_length=30
    )
    recipient = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        help_text=_("Pessoa usuária que recebeu o email"),
        on_delete=models.deletion.CASCADE,
        verbose_name="Pessoa usuária",
        related_name="emails_received",
        null=True,
        blank=True,
    )
    error_message = models.TextField(
        null=True, blank=True, verbose_name=_("Mensagem de erro")
    )

    class Meta:
        app_label = "notification"



import os
from typing import TypedDict
from uuid import uuid4

from django.db import models
from django.utils.deconstruct import deconstructible


class ModelsHooksMixin:
    def save(self, *args, **kwargs):
        is_being_created = self._state.adding
        if is_being_created:
            self.pre_create_instance(*args, **kwargs)
            super().save(*args, **kwargs)
            self.post_create_instance(*args, **kwargs)
        else:
            self.pre_update_instance(*args, **kwargs)
            super().save(*args, **kwargs)
            self.post_update_instance(*args, **kwargs)

    def post_create_instance(self, *args, **kwargs):
        pass

    def post_update_instance(self, *args, **kwargs):
        pass

    def pre_create_instance(self, *args, **kwargs):
        pass

    def pre_update_instance(self, *args, **kwargs):
        pass


class BaseModel(ModelsHooksMixin, models.Model):
    created_at = models.DateTimeField(auto_now_add=True, verbose_name="Criado em")
    updated_at = models.DateTimeField(auto_now=True, verbose_name="Atualizado em")

    class Meta:
        abstract = True


@deconstructible
class GetHasher:
    def __init__(self, size=8):
        self.size = size

    def __call__(self):
        return self._get_hash_pk()

    def _get_hash_pk(self):
        return uuid4().hex[: self.size]


class HashedAutoField(models.CharField):
    def __init__(self, *args, **kwargs):
        size = kwargs.pop("size", 8)
        defaults = {"default": GetHasher(size), "max_length": 32, "editable": False}
        defaults.update(kwargs)
        super().__init__(*args, **defaults)


@deconstructible
class HashedFileName(object):
    def __init__(self, path):
        self.path = os.path.join(path, "%s%s")

    def __call__(self, instance, filename):
        extension = os.path.splitext(filename)[1]
        return self.path % (uuid4(), extension)


class NullableOptions(TypedDict):
    null: bool
    blank: bool


NULLABLE: NullableOptions = {"null": True, "blank": True}



from decimal import Decimal

from django.conf import settings
from django.core.validators import MinValueValidator
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext as _
from phonenumber_field.modelfields import PhoneNumberField
from watson import search as watson

from lacrei_models.client.models import NULLABLE, BaseProfile
from lacrei_models.professional.services.verification import (
    BoardRegistrationNumber,
    PostRegistrationData,
    VerificationStepsService,
)
from lacrei_models.utils.models import BaseModel, HashedAutoField, HashedFileName
from lacrei_models.utils.validators import BRCPFValidator, OnlyAlphabeticValidator

CONTACT_REQUEST_SMS_MESSAGE = "Código Lacrei Saúde: {validation_code}. Não compartilhe."


PROFILE_STATUS = [
    ("pending", _("Informações pendentes")),
    ("in_review", _("Em revisão")),
    ("rejected", _("Rejeitado")),
    ("approved", _("Aprovado")),
]

VERIFICATION_STEPS_MAP = {
    "board_registration_number": {
        "choice_description": "1º etapa - Numero de inscrição",
        "verification_step": BoardRegistrationNumber,
    },
    "post_registration_data": {
        "choice_description": "2º etapa - Pós cadastro",
        "verification_step": PostRegistrationData,
    },
}
VERIFICATION_STEPS_CHOICES = (
    (key, value["choice_description"]) for key, value in VERIFICATION_STEPS_MAP.items()
)


class Profession(BaseModel):
    name = models.CharField(max_length=100)
    search_synonym = models.CharField(max_length=256, default=None, **NULLABLE)

    class Meta:
        verbose_name = _("Profissão")
        verbose_name_plural = _("Profissões")
        app_label = "professional"
        db_table = "lacreisaude_profession"

    def __str__(self):
        return self.name


class WaitingList(BaseModel):
    email = models.EmailField()
    profession = models.ForeignKey(Profession, on_delete=models.PROTECT)
    state = models.ForeignKey("address.State", on_delete=models.PROTECT)

    class Meta:
        verbose_name = _("Lista de espera")
        verbose_name_plural = _("Listas de espera")
        app_label = "professional"
        db_table = "lacreisaude_waitinglist"


class Professional(BaseProfile):
    CPF = "CPF"
    CNPJ = "CNPJ"
    EMAIL = "EMAIL"
    CELULAR = "PHONE"
    EVP = "EVP"
    tipo_chave = [
        (CPF, _("CPF")),
        (CNPJ, _("CNPJ")),
        (EMAIL, _("EMAIL")),
        (CELULAR, _("PHONE")),
        (EVP, _("EVP")),
    ]
    id = HashedAutoField(primary_key=True)
    user = models.OneToOneField("client.User", on_delete=models.PROTECT)
    full_name = models.CharField(
        max_length=200,
        verbose_name=_("Nome completo"),
        validators=[OnlyAlphabeticValidator()],
    )
    about_me = models.TextField(**NULLABLE, verbose_name=_("Sobre mim"))
    profile_status = models.CharField(
        max_length=30,
        choices=PROFILE_STATUS,
        default="pending",
        verbose_name=_("Status do perfil"),
    )
    state = models.ForeignKey(
        "address.State", on_delete=models.PROTECT, verbose_name=_("Estado")
    )
    active = models.BooleanField(
        default=False,
        help_text=(
            _("Define se o perfil está ativo e pode ser alterado pelos administradores")
        ),
        verbose_name=_("Ativo"),
    )
    published = models.BooleanField(
        default=True,
        help_text=(
            _(
                "Define se o perfil está publicado "
                "e pode ser alterado pelo próprio profissional"
            )
        ),
        verbose_name=_("Publicado"),
    )
    document_number = models.CharField(
        max_length=20, **NULLABLE, validators=[BRCPFValidator()], verbose_name=_("CPF")
    )
    profession = models.ForeignKey(
        Profession,
        related_name="professionals",
        on_delete=models.PROTECT,
        verbose_name=_("Profissão"),
    )
    search_synonym = models.CharField(max_length=256, default=None, **NULLABLE)
    board_registration_number = models.CharField(
        max_length=100, verbose_name=_("Número do registro do conselho")
    )
    accepted_privacy_document = models.BooleanField(
        default=False, verbose_name=_("Documento de privacidade aceito")
    )
    privacy_document = models.ForeignKey(
        "client.PrivacyDocument",
        on_delete=models.PROTECT,
        null=True,
        blank=False,
        verbose_name=_("Documento de privacidade"),
    )
    safety_measures = models.CharField(
        max_length=1000,
        **NULLABLE,
        help_text=_("Medidas de segurança do Covid, por exemplo"),
        verbose_name=_("Medidas de segurança"),
    )
    specialty = models.CharField(
        **NULLABLE,
        max_length=250,
        help_text=_("Especialidade da pessoa profissional, como cardiologia"),
        verbose_name=_("Especialidade clínica"),
    )
    specialty_number_rqe = models.CharField(
        **NULLABLE,
        max_length=10,
        help_text=_("Registro de Qualificação de Especialidade (RQE)"),
        verbose_name=_("Registro de Qualificação de Especialidade (RQE)"),
    )
    board_certification_selfie = models.ImageField(
        **NULLABLE,
        upload_to=HashedFileName("board_certification_selfie"),
        verbose_name=_("Selfie do documento de registro do conselho"),
    )
    photo = models.ImageField(
        **NULLABLE,
        upload_to=HashedFileName("professional_photos"),
        verbose_name=_("Foto de perfil"),
    )
    photo_description = models.CharField(
        **NULLABLE,
        max_length=250,
        help_text=_("Descrição da foto de perfil para pessoas com deficiência visual"),
        verbose_name=_("Descrição da foto de perfil"),
    )
    google_calendar_credentials_json = models.JSONField(
        null=True,
        blank=True,
        help_text="Credenciais OAuth 2.0 para a API do Google Calendar.",
    )
    chave_pix = models.CharField(max_length=100, default=None, **NULLABLE)
    tipo_chave_pix = models.CharField(
        max_length=10,
        choices=tipo_chave,
        default=None,
        null=True,
        blank=True,
        verbose_name=_("tipo de chave pix"),
    )
    income_value = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        **NULLABLE,
        verbose_name=_("Renda/Faturamento Mensal"),
        help_text=_(
            "Renda mensal para PF ou faturamento mensal para PJ (obrigatório para criação)"
        ),
    )

    class Meta:
        verbose_name = _("Profissional")
        verbose_name_plural = _("Profissionais")
        app_label = "professional"
        db_table = "lacreisaude_professional"

    def __str__(self):
        return self.full_name

    @property
    def current_step(self):
        current_step = VerificationStepsService(self).current_step()
        if not current_step:
            return None
        return current_step["description"], current_step["internal_message"]


class Clinic(BaseModel):
    id = HashedAutoField(primary_key=True)
    professional = models.OneToOneField(
        Professional,
        on_delete=models.PROTECT,
        related_name="clinic",
        verbose_name=_("Profissional"),
    )
    is_presential_clinic = models.BooleanField(
        default=True, verbose_name=_("Clínica Presencial")
    )
    is_online_clinic = models.BooleanField(
        default=False, verbose_name=_("Clínica Online")
    )
    name = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Nome da Clínica")
    )
    zip_code = models.CharField(max_length=20, **NULLABLE, verbose_name=_("CEP"))
    registered_neighborhood = models.ForeignKey(
        "address.Neighborhood",
        on_delete=models.PROTECT,
        help_text=_("ID do bairro encontrado na API de busca de CEP"),
        **NULLABLE,
        verbose_name=_("ID do bairro"),
    )
    neighborhood = models.CharField(
        max_length=200,
        help_text=_("Nome do bairro"),
        **NULLABLE,
        verbose_name=_("Bairro"),
    )
    city = models.CharField(
        max_length=200,
        help_text=_("Nome da cidade"),
        **NULLABLE,
        verbose_name=_("Cidade"),
    )
    state = models.ForeignKey(
        "address.State", on_delete=models.PROTECT, verbose_name=_("Estado"), **NULLABLE
    )
    address = models.CharField(
        max_length=200,
        help_text=_("Endereço completo, incluindo número"),
        **NULLABLE,
        verbose_name=_("Endereço"),
    )
    address_line2 = models.CharField(
        max_length=200,
        help_text=_("Complemento do endereço"),
        **NULLABLE,
        verbose_name=_("Complemento"),
    )
    phone = PhoneNumberField(
        help_text=_("Telefone da clínica"),
        **NULLABLE,
        verbose_name=_("Telefone da clínica"),
    )
    phone_whatsapp = PhoneNumberField(
        help_text=_("Telefone WhatsApp"), **NULLABLE, verbose_name=_("Whatsapp")
    )
    consult_price = models.DecimalField(
        decimal_places=2,
        max_digits=9,
        validators=[MinValueValidator(Decimal(0.00))],
        help_text=_("Valor da Consulta"),
        **NULLABLE,
        verbose_name=_("Valor da Consulta"),
    )
    duration_minutes = models.PositiveIntegerField(
        validators=[MinValueValidator(1)],
        help_text=_("Duração em minutos da consulta"),
        **NULLABLE,
        verbose_name=_("Duração da consulta"),
    )
    accepts_insurance_providers = models.BooleanField(
        default=False,
        help_text=_("Aceita Convênios?"),
        verbose_name=_("Aceita Convênios"),
    )
    provides_accessibility_standards = models.BooleanField(
        default=False,
        help_text=_("Clínica possui acessibilidade?"),
        verbose_name=_("Clínica possui acessibilidade"),
    )
    online_clinic_phone = PhoneNumberField(
        help_text=_("Telefone da clínica"),
        **NULLABLE,
        verbose_name=_("Telefone da clínica"),
    )
    online_clinic_phone_whatsapp = PhoneNumberField(
        help_text=_("Telefone WhatsApp"), **NULLABLE, verbose_name="WhatsApp"
    )
    online_clinic_consult_price = models.DecimalField(
        decimal_places=2,
        max_digits=9,
        validators=[MinValueValidator(Decimal(0.00))],
        help_text=_("Valor da Consulta"),
        **NULLABLE,
        verbose_name=_("Valor da Consulta online"),
    )
    online_clinic_duration_minutes = models.PositiveIntegerField(
        validators=[MinValueValidator(1)],
        help_text=_("Duração em minutos da consulta"),
        **NULLABLE,
        verbose_name=_("Duração da consulta"),
    )
    online_clinic_accepts_insurance_providers = models.BooleanField(
        default=False,
        help_text=_("Aceita Convênios?"),
        verbose_name=_("Aceita Convênios"),
    )

    def __str__(self):
        return self.name or f"{self._meta.verbose_name} (id: {self.id})"

    class Meta:
        verbose_name = _("Clínica")
        verbose_name_plural = _("Clínicas")
        app_label = "professional"
        db_table = "lacreisaude_clinic"

    def post_create_instance(self, *args, **kwargs):
        # Esta lógica reativa (hook) permanece no modelo
        watson.default_search_engine.update_obj_index(self.professional)

    def post_update_instance(self, *args, **kwargs):
        # Esta lógica reativa (hook) permanece no modelo
        watson.default_search_engine.update_obj_index(self.professional)

    @property
    def full_address(self):
        # Esta lógica de APRESENTAÇÃO permanece no modelo
        parts = [
            self.address or "",
            self.address_line2 or "",
            self.neighborhood or "",
            self.city or "",
            self.state.code if self.state else "",
        ]
        return ", ".join(filter(None, parts)) or "Endereço não cadastrado"


class ClinicOpeningSchedule(BaseModel):
    ONLINE = "online"
    PRESENTIAL = "presential"
    SCHEDULE_TYPES = [
        (ONLINE, _("Online")),
        (PRESENTIAL, _("Presential")),
    ]
    SUN = "sun"
    MON = "mon"
    TUE = "tue"
    WED = "wed"
    THU = "thu"
    FRI = "fri"
    SAT = "sat"
    WEEKDAYS = [
        (SUN, _("Domingo")),
        (MON, _("Segunda-feira")),
        (TUE, _("Terça-feira")),
        (WED, _("Quarta-feira")),
        (THU, _("Quinta-feira")),
        (FRI, _("Sexta-feira")),
        (SAT, _("Sábado")),
    ]
    WEEKDAYS_MAP = {k: v for k, v in WEEKDAYS}
    id = HashedAutoField(primary_key=True)
    schedule_type = models.CharField(
        max_length=15,
        choices=SCHEDULE_TYPES,
        default="presential",
        verbose_name=_("Tipo de agendamento"),
    )
    weekday = models.CharField(
        max_length=3, choices=WEEKDAYS, verbose_name="Dia da semana"
    )
    clinic = models.ForeignKey(
        Clinic, on_delete=models.CASCADE, related_name="opening_schedules"
    )
    opening_time = models.TimeField(verbose_name="Horário de Abertura")
    closing_time = models.TimeField(verbose_name="Horário de fechamento")

    class Meta:
        verbose_name = _("Horário de abertura da clínica")
        verbose_name_plural = _("Horários de abertura da clínica")
        app_label = "professional"
        db_table = "lacreisaude_clinicopeningschedule"


class ProfessionalReview(BaseModel):
    id = HashedAutoField(primary_key=True)
    reviewed_by = models.ForeignKey(
        "client.User", on_delete=models.PROTECT, verbose_name=_("Revisado por")
    )
    professional = models.ForeignKey(
        Professional,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
        verbose_name=_("Profissional"),
    )
    professional_name = models.CharField(
        max_length=250, verbose_name=_("Nome do profissional")
    )
    profession = models.ForeignKey(
        Profession,
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        verbose_name=_("Profissão"),
    )
    internal_note = models.TextField(verbose_name=_("Nota interna"))
    status = models.CharField(
        max_length=20,
        choices=[
            ("approved", "Aprovado"),
            ("rejected", "Rejeitado"),
        ],
    )
    rejected_professional_data = models.JSONField(
        null=True,
        blank=True,
    )
    step = models.CharField(
        max_length=30,
        choices=VERIFICATION_STEPS_CHOICES,
        null=True,
        verbose_name=_("Etapas"),
    )

    def __str__(self):
        return self.professional_name

    class Meta:
        verbose_name = _("Revisão de profissional")
        verbose_name_plural = _("Revisão de profissionais")
        app_label = "professional"
        db_table = "lacreisaude_professionalreview"


class ContactRequest(BaseModel):
    id = HashedAutoField(primary_key=True)
    requester = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        help_text=_("Pessoa usuária solicitando o contato da profissional"),
        on_delete=models.deletion.PROTECT,
        verbose_name=_("Solicitante"),
    )
    requester_ip_address = models.GenericIPAddressField(verbose_name=_("Endereço IP"))
    requester_phone_number = PhoneNumberField(verbose_name=_("Telefone solicitante"))
    requester_user_agent = models.TextField(verbose_name=_("User Agent"))
    professional = models.ForeignKey(
        Professional, on_delete=models.deletion.PROTECT, verbose_name=_("Profissional")
    )
    validation_code = models.CharField(
        max_length=6, verbose_name=_("Código de validação")
    )
    code_confirmed = models.BooleanField(
        default=False, verbose_name=_("Código confirmado")
    )
    expires_at = models.DateTimeField(verbose_name=_("Expira em"))
    created_at = models.DateTimeField(default=timezone.now, verbose_name=_("Criado em"))

    class Meta:
        verbose_name = _("Solicitação de Contato")
        verbose_name_plural = _("Solicitações de Contato")
        app_label = "professional"
        db_table = "lacreisaude_contactrequest"

    EXPIRES_IN_DAYS = 7

    def __str__(self) -> str:
        return f"{self.requester} -> {self.professional}"

    @property
    def sms_message(self):
        return CONTACT_REQUEST_SMS_MESSAGE.format(validation_code=self.validation_code)


class Complaint(BaseModel):
    id = HashedAutoField(primary_key=True)
    complaint_types = models.CharField(
        max_length=200, null=True, blank=True, verbose_name=_("Tipos de comportamento")
    )
    other_complaint_type = models.CharField(
        max_length=200,
        null=True,
        blank=True,
        verbose_name="Outros tipos de comportamento",
    )
    incident_date = models.DateField(verbose_name=_("Data do ocorrido"))
    incident_time = models.TimeField(verbose_name=_("Horário do ocorrido"))
    reported_by = models.ForeignKey(
        "client.User", on_delete=models.PROTECT, verbose_name=_("Reportado por")
    )
    description = models.TextField(verbose_name=_("Descrição do ocorrido"))
    professional = models.ForeignKey(
        Professional, on_delete=models.deletion.PROTECT, verbose_name=_("Profissional")
    )

    class Meta:
        verbose_name = _("Denúncia")
        verbose_name_plural = _("Denúncias")
        app_label = "professional"
        db_table = "lacreisaude_complaint"


class ProfessionalSubaccount(BaseModel):
    id = HashedAutoField(primary_key=True)
    professional = models.OneToOneField(
        "professional.Professional",
        on_delete=models.CASCADE,
        related_name="subaccount",
        verbose_name=_("Profissional"),
    )
    asaas_id = models.CharField(
        max_length=100, unique=True, verbose_name=_("ID no Asaas")
    )
    onboarding_url = models.URLField(
        max_length=500, **NULLABLE, verbose_name=_("URL de Onboarding de Documentos")
    )
    status_general = models.CharField(
        max_length=50, default="AWAITING_APPROVAL", verbose_name=_("Status Geral")
    )
    status_commercial_info = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Info. Comerciais"),
    )
    status_bank_account = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Conta Bancária"),
    )
    status_documentation = models.CharField(
        max_length=50,
        default="AWAITING_APPROVAL",
        verbose_name=_("Status Documentação"),
    )

    class Meta:
        verbose_name = _("Subconta de Profissional (Asaas)")
        verbose_name_plural = _("Subcontas de Profissionais (Asaas)")
        app_label = "professional"
        db_table = "lacreisaude_professionalsubaccount"

    def __str__(self):
        return f"Subconta de {self.professional.full_name} ({self.asaas_id})"


class SubaccountDocument(BaseModel):
    id = HashedAutoField(primary_key=True)
    subaccount = models.ForeignKey(
        ProfessionalSubaccount,
        on_delete=models.CASCADE,
        related_name="documents",
        verbose_name=_("Subconta"),
    )
    document_type = models.CharField(
        max_length=100,
        verbose_name=_("Tipo de Documento"),
        help_text=_("Ex: IDENTIFICATION, SOCIAL_CONTRACT"),
    )
    file = models.FileField(
        upload_to=HashedFileName("subaccount_documents"),
        verbose_name=_("Arquivo do Documento"),
    )
    asaas_id = models.CharField(
        max_length=100, unique=True, verbose_name=_("ID do Documento no Asaas")
    )
    status = models.CharField(
        max_length=50, default="pending", verbose_name=_("Status do Documento")
    )
    rejection_reason = models.TextField(
        **NULLABLE, verbose_name=_("Motivo da Rejeição")
    )

    class Meta:
        verbose_name = _("Documento de Subconta")
        verbose_name_plural = _("Documentos de Subcontas")
        app_label = "professional"
        db_table = "lacreisaude_subaccountdocument"

    def __str__(self):
        return f"Documento {self.document_type} de {self.subaccount.professional.full_name}"


class LegacyProfessionalData(BaseModel):
    """
    Modelo para armazenar dados de diversidade LEGADOS do Professional,
    espelhando a estrutura (Foreign Keys names + CharField 'Other') do sistema antigo.
    """

    professional = models.OneToOneField(
        "Professional",
        on_delete=models.CASCADE,
        related_name="legacy_social_data",
        verbose_name=_("Profissional Original"),
    )

    ethnic_group_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Grupo Étnico (Nome Legado)"),
    )

    gender_identity_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Identidade de Gênero (Nome Legado)"),
    )

    sexual_orientation_name = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Orientação Sexual (Nome Legado)"),
    )

    pronoun_text = models.CharField(
        max_length=50, **NULLABLE, verbose_name=_("Pronome (Texto Legado)")
    )

    article_text = models.CharField(
        max_length=1, **NULLABLE, verbose_name=_("Artigo (Texto Legado)")
    )

    disability_types_names = models.TextField(
        **NULLABLE, verbose_name=_("Tipos de Deficiência (Nomes, LEGADO)")
    )

    other_ethnic_group = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Grupo Étnico (LEGADO)")
    )
    other_gender_identity = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Outra Identidade de Gênero (LEGADO)"),
    )
    other_sexual_orientation = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outra Orientação Sexual (LEGADO)")
    )
    other_pronoun = models.CharField(
        max_length=100, **NULLABLE, verbose_name=_("Outro Pronome (LEGADO)")
    )
    other_disability_types = models.CharField(
        max_length=100,
        **NULLABLE,
        verbose_name=_("Outro Tipos de deficiência (LEGADO)"),
    )
    other_article = models.CharField(
        max_length=1, **NULLABLE, verbose_name=_("Outro Artigo (LEGADO)")
    )

    class Meta:
        verbose_name = _("Dados Sociais Legados do Profissional")
        verbose_name_plural = _("Dados Sociais Legados dos Profissionais")
        app_label = "professional"
        db_table = "lacreisaude_legacyprofessionaldata"
